package eu.jonahbauer.wizard.client.libgdx.state;

import eu.jonahbauer.wizard.client.libgdx.Client;
import eu.jonahbauer.wizard.client.libgdx.screens.LoadingScreen;
import eu.jonahbauer.wizard.client.libgdx.util.SavedData;
import eu.jonahbauer.wizard.common.messages.data.SessionData;
import eu.jonahbauer.wizard.common.messages.server.*;
import eu.jonahbauer.wizard.common.model.Configuration;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

import java.util.Optional;
import java.util.UUID;

@Log4j2
@Getter
@RequiredArgsConstructor
public final class AwaitingJoinSession extends Awaiting {

    private final UUID session;
    private final String sessionName;
    private final Configuration configuration;
    private final String playerName;

    @Override
    public Optional<ClientState> onEnter(Client client) {
        log.info("Waiting for acknowledgment...");
        client.getGame().setScreen(new LoadingScreen(client.getGame(), "menu.loading.joining_session"));
        return super.onEnter(client);
    }

    @Override
    public Optional<ClientState> onMessage(Client client, ServerMessage message) {
        if (message instanceof SessionJoinedMessage joined) {
            if (session != null && !session.equals(joined.getSession())) {
                return super.onMessage(client, message);
            } else {
                log.info("There are {} players in this session.", joined.getPlayers().size());
                log.info("Your uuid is {}.", joined.getPlayer());
                log.info("Your secret is {}.", joined.getSecret());

                client.getGame().storage.credentials = new SavedData.SessionCredentials(
                        joined.getSession(),
                        joined.getPlayer(),
                        joined.getSecret()
                );

                return Optional.of(new Session(
                        new SessionData(joined.getSession(), sessionName, -1, configuration),
                        joined.getPlayers(),
                        joined.getPlayer()
                ));
            }
        } else if (message instanceof NackMessage nack) {
            switch (nack.getCode()) {
                case NackMessage.GAME_ALREADY_STARTED -> log.error("Game has already started.");
                case NackMessage.SESSION_FULL -> log.error("The session is full.");
                case NackMessage.SESSION_NOT_FOUND -> log.error("Session not found.");
                case NackMessage.NAME_TAKEN -> log.error("Name already taken.");
                default -> { return super.onMessage(client, message); }
            }
            return Optional.of(new AwaitingJoinLobby());
        } else if (message instanceof SessionModifiedMessage || message instanceof SessionRemovedMessage) {
            // drop
            log.debug("Dropped message {}.", message);
            return Optional.empty();
        } else {
            return super.onMessage(client, message);
        }
    }
}
